from semanticnetwork import TangledSemanticNetwork
import csv

class KnowledgeBase(object):
    def __init__(self, initial_bases=[]):
        self._bases = {}
        for b in initial_bases:
            self.add_base(b)

    def add_base(self, base):
        if not base in self._bases:
            self._bases[base] = TangledSemanticNetwork()
            
        
    def get_base(self, base):
        return self._bases[base]

    def del_base(self, base):
        del self._bases[base]

    def __getattribute__(self, a):
        if a[0] == '_' or not a in self._bases:
            return super(KnowledgeBase, self).__getattribute__(a)
        else:
            return self.get_base(a)


    def __getitem__(self, i):
        return self._bases[i]

    def __contains__(self, c):
        return c in self._bases
    
def bootstrap(files):
    """
    Return a knowledge base loaded with the information given in the file.
    File format is a simple CSV:
    base, parent, child
    language, partofspeech, noun
    language, partofspeech, verb
    science, oxygen, element
    etc.
    """
    ret = KnowledgeBase()

    for f in files:
        reader = csv.DictReader(open(f, "r"))

        for line in reader:
            if not line['base'] in ret:
                ret.add_base(line['base'])
            ret[line['base']][line['child'].lower()] = line['parent'].lower()

        return ret
