import sys
from os import path

# add modules to the import path
modules_path = path.dirname(path.abspath(__file__)) + path.sep + ".."
sys.path.append(modules_path)

from cmsc421p5.main import process_line
from cmsc421p5.knowledgebase import KnowledgeBase
import unittest
import pytest


class TestTangledSemanticNetwork(unittest.TestCase):
    
    def setUp(self):
        pass

    def load_kb(self):
        kb = KnowledgeBase(['language', 'attributes', 'category'])
        kb.language['noun'] = 'partofspeech'
        kb.language['adjective'] = 'partofspeech'
        kb.language['verb'] = 'partofspeech'
        kb.language['dog'] = 'noun'
        kb.language['green'] = 'adjective'
        kb.language['black'] = 'adjective'
        kb.language['is'] = 'verb'
        kb.language['article'] = 'partofspeech'
        kb.language['the'] = 'article'
        return kb

    def test_add_attributes(self):
        kb = self.load_kb()
        process_line(kb, ['the', 'dog', 'is', 'green'])
        self.assertIn('dog', kb.attributes)
        self.assertIn('green', kb.attributes['dog'])
        
    def test_question_attributes(self):
        kb = self.load_kb()
        (ret, out) = process_line(kb, ['the', 'dog', 'is', 'green'])
        self.assert_(ret)
        self.assertIn('dog', kb.attributes)
        self.assertIn('green', kb.attributes['dog'])
        (ret, out) = process_line(kb, ['what', 'color', 'is', 'the', 'dog'])
        self.assert_(ret)
        self.assertIn('green', out)

    def test_question_categories(self):
        kb = self.load_kb()
        (ret, out) = process_line(kb, ['dogs', 'are', 'pets'])
        self.assert_(ret)
        self.assertIn('dog', kb.category)
        self.assertIn('pet', kb.category['dog'])
        (ret, out) = process_line(kb, ['are', 'dogs', 'pets'])
        self.assert_(ret)
        self.assertIn('yes', map(lambda s: s.lower(), out))
        
        


if __name__ == "__main__":
    pytest.main()
