import sys
from os import path

modules_path = path.dirname(path.abspath(__file__)) + path.sep + ".."
sys.path.append(modules_path)

from cmsc421p5.semanticnetwork import TangledSemanticNetwork
import cmsc421p5.knowledgebase as kb
from cmsc421p5.parser import Sentence, ParseError
import unittest
import pytest

class TestParser(unittest.TestCase):
    def setUp(self):
        fn = path.join(path.dirname(__file__), '..', 'kb-basic.csv')
        self.kb = kb.bootstrap([fn])
    
    def test_simple_declarative(self):
        try: 
            s = Sentence("Fido is a dog", self.kb)
            self.assertTrue(s.np.proper_noun == "fido")
            Sentence("John is colorless", self.kb)
            Sentence("the dog is green", self.kb)
            Sentence("The dog is not a cat", self.kb)
            Sentence("Colorless green ideas sleep furiously", self.kb)
            print s.np
        except ParseError:
            self.fail("Parse error")

    def test_stemmed_sentence(self):
        try: 
            Sentence("Sue sleeps on the mat", self.kb)
            Sentence("Sue is a mat", self.kb)
            Sentence("Sue slept on the mat", self.kb)
            Sentence("Sue sleeps on the mats", self.kb)
            Sentence("Does Sue sleep on the mat", self.kb)
        except ParseError:
            self.fail("Parse error")

    def test_interrogative_pronoun(self):
        try:
            Sentence("What is a dog", self.kb)
            Sentence("Who eats the dog", self.kb)
            Sentence("Where does the dog eat", self.kb)
        except ParseError:
            self.fail("Parse error")

    def test_interrogative_verb(self):
        try:
            Sentence("Does the dog sleep", self.kb)
            Sentence("Is Sue a dog", self.kb)
        except ParseError:
            self.fail("Parse error")

    def test_int_pronoun_verb(self):
        try:
            Sentence("How does the dog sleep", self.kb)
            Sentence("Where does Fido sleep", self.kb)
        except ParseError:
            self.fail("Parse error")

    def test_simple_imperative(self):
        try:
            Sentence("Eat the dog", self.kb)
        except ParseError:
            self.fail("Parse error")

    def test_with_punctuation(self):
        try:
            Sentence("Sue is a mat.", self.kb)
            Sentence("Where does Fido sleep?", self.kb)
            Sentence("Eat the dog!", self.kb)
        except ParseError:
            self.fail("Parse error")

    def test_nonsense(self):
        self.assertRaises(ParseError,
            Sentence, "Mat sue is a!", self.kb)
        self.assertRaises(ParseError,
            Sentence, "Does where sleep a the?", self.kb)
        
    def test_almost_sentence(self):
        self.assertRaises(ParseError,
            Sentence, "Sue sleeps green", self.kb)
        self.assertRaises(ParseError,
            Sentence, "John not green", self.kb)
    
    def test_complicated_sentence(self):
        try:
            Sentence("Why does Sue sleep furiously?", self.kb)
        except ParseError:
            self.fail("Parse error")

    def test_connected_sentence(self):
        try:
            s = Sentence("Sue has a dog and Sue has a cat", self.kb)
            self.assertEquals(s.sentence.np.proper_noun, "sue")
        except ParseError:
            self.fail("ParseError")

    def test_int_verb_declarative(self):
        try:
            s = Sentence("Sue does have a dog", self.kb)
            self.assertEquals(s.vp.vp.verb, "have")
            s = Sentence("The dog should eat the cat", self.kb)
        except ParseError:
            self.fail("ParseError")
         
if __name__ == "__main__":
    pytest.main()
