import sys
from os import path

# add modules to the import path
modules_path = path.dirname(path.abspath(__file__)) + path.sep + ".."
sys.path.append(modules_path)

from cmsc421p5.knowledgebase import TangledSemanticNetwork
import unittest
import pytest


class TestTangledSemanticNetwork(unittest.TestCase):
    
    def setUp(self):
        pass

    def test_contains(self):
        tsn = TangledSemanticNetwork("root")
        self.assertTrue("root" in tsn)
        tsn['root'] = "foo"
        self.assertTrue("foo" in tsn)
        tsn.link("foo", "bar")
        self.assertTrue("bar" in tsn)

    def test_get(self):
        tsn = TangledSemanticNetwork("root")
        tsn['root'] = 'toor'
        tsn['root'] = 'foobar'
        tsn['root'] = 'barfoo'
        val = tsn['root']
        val.sort()
        self.assertEquals(['barfoo', 'foobar', 'toor'], val)

    def test_unlink(self):
        tsn = TangledSemanticNetwork('root')
        tsn['root'] = 'foo'
        self.assertTrue('foo' in tsn)
        tsn.unlink('root', 'foo')
        self.assertFalse('foo' in tsn)

    def test_paths(self):
        tsn = TangledSemanticNetwork('root')
        tsn['root'] = 'foo'
        paths = tsn.paths('root', 'foo')
        self.assertTrue(reduce(lambda a,n: a and type(n) == list, paths, 
                               True))
        self.assertEqual(paths, [['root', 'foo']])
        tsn['root'] = 'bar'
        tsn['bar'] = 'end'
        tsn['foo'] = 'end'
        self.assertEqual(tsn.paths('root', 'end'),
                         [['root', 'foo', 'end'],
                          ['root', 'bar', 'end']])
        tsn['root'] = 'no_path_to_end'
        self.assertEqual(tsn.paths('root', 'end'),
                         [['root', 'foo', 'end'],
                          ['root', 'bar', 'end']])

    def test_above_below(self):
        tsn = TangledSemanticNetwork()
        tsn['cat'] = 'pet'
        tsn['dog'] = 'pet'
        tsn['pet'] = 'animal'
        self.assert_(tsn.succeeds('cat', 'animal'))
        self.assert_(tsn.succeeds('cat', 'pet'))
        self.assert_(tsn.succeeds('pet', 'pet'))
        self.assert_(tsn.precedes('animal', 'pet'))
        self.assert_(tsn.precedes('animal', 'cat'))
        self.assert_(tsn.precedes('animal', 'animal'))
        self.assertFalse(tsn.precedes('cat', 'animal'))
        self.assertFalse(tsn.precedes('cat', 'pet'))
        self.assertEqual(tsn.succeeds('cat'), set(['pet']))
        self.assertEqual(tsn.precedes('pet'), set(['cat', 'dog']))

if __name__ == "__main__":
    pytest.main()
